#!/usr/bin/env python3
"""
Entry point for running teradata_mcp_server as a module.

This allows the package to be executed with:
    python -m teradata_mcp_server
"""

from . import main

if __name__ == "__main__":
    main()
